function [apScore,precision,reccall,isTp,isFp,isMissed,gtAssignment,maxOverlaps,numOfgt] = computePRCurve2D(classname,predictedBbs,imageIds,groundTruthBbs,isDifficult)
    if length(groundTruthBbs) ~= length(isDifficult), error('inconsistent difficulty size.'); end
    
    gtAssignment = zeros(size(predictedBbs,1),1);
    
    % pick ground truth of current class
    ConstsDetection;
    isMissed = false(numel(groundTruthBbs),1);
    [~,classId] = ismember(classname,consts.classNames);
    isSameClass = [groundTruthBbs.classId] == classId;
    numOfgt =sum(isSameClass);
    groundTruthBbs = groundTruthBbs(isSameClass);
    isDifficult = logical(isDifficult(isSameClass));

    P = size(predictedBbs,1);
    G = numel(groundTruthBbs);
    
    % sort all detections
    [~,sortIdx] = sort(predictedBbs(:,5),'descend');
    predictedBbs = predictedBbs(sortIdx,:);
    imageIds =imageIds(sortIdx,:);
    % threshold overlap
    tic;
    predictedBbs =predictedBbs(:,1:4);
    allOverlaps = bb2dOverlap(predictedBbs,groundTruthBbs);
    toc;
    onSameImage = bsxfun(@eq,imageIds(:),[groundTruthBbs.imageNum]);
    allOverlaps(~onSameImage) = 0;
    [maxOverlaps,gtIdx] = max(allOverlaps,[],2);
    isOverlapping = maxOverlaps >= 0.5;
    gtIdx(~isOverlapping) = 0;
    
    % Assign ground truth to the best matched detection
    [uniqueGtIdx,firstAssignment,~] = unique(gtIdx,'first');
    isFirstAssignment = false(P,1);
    isFirstAssignment(firstAssignment) = true;
    isFirstAssignment = isFirstAssignment & gtIdx>0;
    
    % Get GT assignment of each TP detection
    tmpGtAssignment = zeros(P,1);
    tmpGtAssignment(firstAssignment) = uniqueGtIdx;
    tmp = find(isSameClass);
    tmpGtAssignment(tmpGtAssignment>0) = tmp(tmpGtAssignment(tmpGtAssignment>0));
    gtAssignment(sortIdx) = tmpGtAssignment;
    

    
    % get unassigned ground truth bounding boxes
    tmpIsMissed = true(G,1);
    tmpIsMissed(setdiff(uniqueGtIdx,0)) = false;
    isMissed(isSameClass) = tmpIsMissed;
    
    % assign detection labels
    tp = isFirstAssignment & isOverlapping;
    fp = ~tp;
    dc = gtIdx ~= 0 & isDifficult(max(1,gtIdx));
    tp(dc) = false;
    fp(dc) = false;
    isTp = false(numel(tp),1);
    isTp(sortIdx) = tp;
    isFp = false(numel(fp),1);
    isFp(sortIdx) = fp;
    
    % Compute precision/recall
    sumFp = cumsum(double(fp));
    sumTp = cumsum(double(tp));
    reccall = sumTp / sum(~isDifficult);
    precision = sumTp ./ (sumFp + sumTp);
    precision(isnan(precision)) =0;
    apScore = get_average_precision(precision, reccall);
end

function allOverlaps = bb2dOverlap(predictedBbs,groundTruthBbs)
if size(groundTruthBbs,2)==1
    gtbb2D =reshape([groundTruthBbs.gtBb2D],4,[])';
else
    gtbb2D =gtbb2D;
end
gtbb2D(:,3:4) = gtbb2D(:,1:2)+gtbb2D(:,3:4);
predictedBbs(:,3:4) =predictedBbs(:,1:2)+predictedBbs(:,3:4);
allOverlaps =zeros(length(predictedBbs),length(groundTruthBbs));
% for n =1:length(predictedBbs),
%     for m =1:length(groundTruthBbs)
%         allOverlaps(n,m) =overlapSingle(predictedBbs(n,:),gtbb2D(m,:));
%     end
% end
for ngt=1:length(groundTruthBbs)
     allOverlaps(:,ngt) = overlapArray(predictedBbs,gtbb2D(ngt,:));
end
end
function out= overlapSingle(bb1,bb2)
    if bb1(1) > bb2(3),out =0 ;return ;end
    if (bb1(2) > bb2(4)),out =0 ; return ; end
    if (bb1(3) < bb2(1)),out =0 ;return ; end
    if (bb1(4) < bb2(2)),out =0 ;return ; end

    b=min(bb1(3:4),bb2(3:4))-max(bb1(1:2),bb2(1:2));
    b(b<0)=0;
    intersection =b(1).*b(2);
    area1=(bb1(3)-bb1(1)+1)*(bb1(4)-bb1(2)+1);
    area2=(bb2(3)-bb2(1)+1)*(bb2(4)-bb2(2)+1);
    out = intersection/(area1 + area2 - intersection);
end

function out= overlapArray(bb1,bb2)
    b= [min(bb1(:,3),bb2(3)),min(bb1(:,4),bb2(4))]-...
         [max(bb1(:,1),bb2(1)),max(bb1(:,2),bb2(2))];
    b(b<0)=0;
    intersection =b(:,1).*b(:,2);
    area1=(bb1(:,3)-bb1(:,1)+1).*(bb1(:,4)-bb1(:,2)+1);
    area2=(bb2(:,3)-bb2(:,1)+1).*(bb2(:,4)-bb2(:,2)+1);
    out = intersection./(area1 + area2 - intersection);
    out(bb1(:,1) > bb2(3),:) =0;
    out(bb1(:,2) > bb2(4),:) =0;
    out(bb1(:,3) < bb2(1),:) =0;
    out(bb1(:,4) < bb2(2),:) =0;
end


function [rec,prec,ap] = VOCevaldet(VOCopts,id,cls,draw)

% load test set

cp=sprintf(VOCopts.annocachepath,VOCopts.testset);
if exist(cp,'file')
    fprintf('%s: pr: loading ground truth\n',cls);
    load(cp,'gtids','recs');
else
    [gtids,t]=textread(sprintf(VOCopts.imgsetpath,VOCopts.testset),'%s %d');
    for i=1:length(gtids)
        % display progress
        if toc>1
            fprintf('%s: pr: load: %d/%d\n',cls,i,length(gtids));
            drawnow;
            tic;
        end

        % read annotation
        recs(i)=PASreadrecord(sprintf(VOCopts.annopath,gtids{i}));
    end
    save(cp,'gtids','recs');
end

fprintf('%s: pr: evaluating detections\n',cls);

% hash image ids
hash=VOChash_init(gtids);
        
% extract ground truth objects

npos=0;
gt(length(gtids))=struct('BB',[],'diff',[],'det',[]);
for i=1:length(gtids)
    % extract objects of class
    clsinds=strmatch(cls,{recs(i).objects(:).class},'exact');
    gt(i).BB=cat(1,recs(i).objects(clsinds).bbox)';
    gt(i).diff=[recs(i).objects(clsinds).difficult];
    gt(i).det=false(length(clsinds),1);
    npos=npos+sum(~gt(i).diff);
end

% load results
[ids,confidence,b1,b2,b3,b4]=textread(sprintf(VOCopts.detrespath,id,cls),'%s %f %f %f %f %f');
BB=[b1 b2 b3 b4]';

% sort detections by decreasing confidence
[sc,si]=sort(-confidence);
ids=ids(si);
BB=BB(:,si);

% assign detections to ground truth objects
nd=length(confidence);
tp=zeros(nd,1);
fp=zeros(nd,1);
tic;
for d=1:nd
    % display progress
    if toc>1
        fprintf('%s: pr: compute: %d/%d\n',cls,d,nd);
        drawnow;
        tic;
    end
    
    % find ground truth image
    i=VOChash_lookup(hash,ids{d});
    if isempty(i)
        error('unrecognized image "%s"',ids{d});
    elseif length(i)>1
        error('multiple image "%s"',ids{d});
    end

    % assign detection to ground truth object if any
    bb=BB(:,d);
    ovmax=-inf;
    for j=1:size(gt(i).BB,2)
        bbgt=gt(i).BB(:,j);
        bi=[max(bb(1),bbgt(1)) ; max(bb(2),bbgt(2)) ; min(bb(3),bbgt(3)) ; min(bb(4),bbgt(4))];
        iw=bi(3)-bi(1)+1;
        ih=bi(4)-bi(2)+1;
        if iw>0 & ih>0                
            % compute overlap as area of intersection / area of union
            ua=(bb(3)-bb(1)+1)*(bb(4)-bb(2)+1)+...
               (bbgt(3)-bbgt(1)+1)*(bbgt(4)-bbgt(2)+1)-...
               iw*ih;
            ov=iw*ih/ua;
            if ov>ovmax
                ovmax=ov;
                jmax=j;
            end
        end
    end
    % assign detection as true positive/don't care/false positive
    if ovmax>=VOCopts.minoverlap
        if ~gt(i).diff(jmax)
            if ~gt(i).det(jmax)
                tp(d)=1;            % true positive
		gt(i).det(jmax)=true;
            else
                fp(d)=1;            % false positive (multiple detection)
            end
        end
    else
        fp(d)=1;                    % false positive
    end
end

% compute precision/recall
fp=cumsum(fp);
tp=cumsum(tp);
rec=tp/npos;
prec=tp./(fp+tp);

ap=VOCap(rec,prec);

if draw
    % plot precision/recall
    plot(rec,prec,'-');
    grid;
    xlabel 'recall'
    ylabel 'precision'
    title(sprintf('class: %s, subset: %s, AP = %.3f',cls,VOCopts.testset,ap));
end
end