function visualizeDetection_noGT(modelImageFolder,pointFolder,dataFolder,testIdRange,modelName,nExamples,outputpath,viewScale,otherNegfiles)
    showModel =0;
    % analyze model name
    [rIdx,rEIdx] = regexp([modelName '_'],'_r-?[.\d]+_');
    [sIdx,sEIdx] = regexp([modelName '_'],'_s-?[.\d]+_');
    [dIdx,dEIdx] = regexp([modelName '_'],'_d-?[.\d]+_');
    [tIdx,tEIdx] = regexp([modelName '_'],'_t-?[.\d]+_');
    if length(rIdx) > 1 || length(sIdx) > 1 || length(dIdx) > 1 || length(tIdx) > 1, error('invalid model name'); end
    instanceNameEndIdx = length(modelName);
    if ~isempty(rIdx), instanceNameEndIdx = min(instanceNameEndIdx,rIdx-1); end
    if ~isempty(sIdx), instanceNameEndIdx = min(instanceNameEndIdx,sIdx-1); end
    if ~isempty(dIdx), instanceNameEndIdx = min(instanceNameEndIdx,dIdx-1); end
    if ~isempty(tIdx), instanceNameEndIdx = min(instanceNameEndIdx,tIdx-1); end
    rString = modelName(rIdx+1:rEIdx-1);
    sString = modelName(sIdx+1:sEIdx-1);
    dString = modelName(dIdx+1:dEIdx-1);
    tString = modelName(tIdx+1:tEIdx-1);
    instanceName = modelName(1:instanceNameEndIdx);
    idx = regexp(instanceName,'_(\d)');
    
    if isempty(idx),
        categName = instanceName;
        instanceString = [];
    else
        categName = instanceName(1:idx-1);
        instanceString = instanceName(idx+1:end);
    end
    
    % find instances that matches model name
    instanceFolders = dir([dataFolder '/' categName '/']);
    instanceFolders = instanceFolders([instanceFolders.isdir]);
    keep = false(length(instanceFolders),1);
    for i = 1:length(instanceFolders),
        keep(i) = ~isempty(regexp(instanceFolders(i).name, instanceName, 'once'));
    end
    instanceFolders = instanceFolders(keep);
    if isempty(instanceFolders), error('no matched instance found'); end
    
    % find views that matches model name, and load mat files
    allModelNames = cell(0,0);
    allbb3d = [];
    allbb3dtight = [];
    allbb3d_fea =[];
    allBb2dDraw = [];
    allTestImgIds = [];
    allInstancetNames = cell(0,0);
    allViewNames = cell(0,0);
    for i = 1:length(instanceFolders),
         viewFoldersCurr = dir([dataFolder '/' categName '/' instanceFolders(i).name '/']);
         viewFoldersCurr = viewFoldersCurr([viewFoldersCurr.isdir]);
        
        % pick view folders that matches input rotation and scale
        keep = false(length(viewFoldersCurr),1);
        for j = 1:length(viewFoldersCurr),
            nameToMatch = [viewFoldersCurr(j).name '_'];
            keep(j) = ~isempty(regexp(nameToMatch, instanceName, 'once'));
            keep(j) = keep(j) && (isempty(rString) || ~isempty(regexp(nameToMatch, ['_' rString '_'], 'once')));
            keep(j) = keep(j) && (isempty(sString) || ~isempty(regexp(nameToMatch, ['_' sString '_'], 'once')));
            keep(j) = keep(j) && (isempty(dString) || ~isempty(regexp(nameToMatch, ['_' dString '_'], 'once')));
            keep(j) = keep(j) && (isempty(tString) || ~isempty(regexp(nameToMatch, ['_' tString '_'], 'once')));
        end
        viewFoldersCurr = viewFoldersCurr(keep);
        
        % load mat files
        parfor j = 1:length(viewFoldersCurr),
            fprintf('loading results of %s...\n',viewFoldersCurr(j).name);
            
            matFiles = dir([dataFolder '/' categName '/' instanceFolders(i).name '/' viewFoldersCurr(j).name '/*.mat']);
            allModelNames = [allModelNames; {viewFoldersCurr(j).name}];
            testIdMin = min(testIdRange);
            testIdMax = max(testIdRange);
            for k = 1:length(matFiles),
                % parse mat file name
                tmp = regexp(matFiles(k).name,'-','split');
                testImgIdCurr = str2double(tmp{1}(5:end));
                if testImgIdCurr < testIdMin || testImgIdCurr > testIdMax, continue; end
                
                % load mat file
                tmp = load([dataFolder '/' categName '/' instanceFolders(i).name '/' viewFoldersCurr(j).name '/' matFiles(k).name]);
                bb3dCurr =tmp.result.bb3d;
                bb2dDrawCurr = tmp.result.bb2dDraw;
                allbb3d =[allbb3d;bb3dCurr];
                allbb3d_fea = [allbb3d_fea;tmp.result.bbw_3d_f];
                allbb3dtight = [allbb3dtight; tmp.result.bbtight3d;];
                
                allBb2dDraw = [allBb2dDraw; bb2dDrawCurr];
                allTestImgIds = [allTestImgIds; repmat(testImgIdCurr,[size(bb3dCurr,1) 1])];
                allInstancetNames = [allInstancetNames; repmat({instanceFolders(i).name},[size(bb3dCurr,1) 1])];
                allViewNames = [allViewNames; repmat({viewFoldersCurr(j).name},[size(bb3dCurr,1) 1])];
            end
        end
    end
    if isempty(allbb3d), error('no matched mat files found'); end
        
        % sort detections according to confidence
        fprintf('sorting detections...\n');
        [~,idx] = sort(allBb2dDraw(:,17),'descend');
        allbb3d = allbb3d(idx,:);
        allbb3dtight = allbb3dtight(idx);
        allbb3d_fea = allbb3d_fea(idx,:);
        allBb2dDraw = allBb2dDraw(idx,:);
        allTestImgIds = allTestImgIds(idx);
        allInstancetNames = allInstancetNames(idx);
        allViewNames = allViewNames(idx);
        % non-max suppression
        uniqueTestImgIds = unique(allTestImgIds);
        discard = false(size(allbb3d,1),1);
        for i = 1:length(uniqueTestImgIds),
            idx = find(allTestImgIds == uniqueTestImgIds(i));
            pick = nmsMe_3d([allbb3d(idx,1:3),allbb3d(idx,1:3)+allbb3d(idx,4:6),allbb3d(idx,7)],0.3);
            discardCurr = false(size(allbb3d,1),1);
            discardCurr(idx(setdiff(1:length(idx),pick))) = true;
            discard = discard | discardCurr;
        end
        allbb3d(discard,:) = [];
        allbb3dtight(discard,:) = [];
        allbb3d_fea(discard,:) = [];
        allBb2dDraw(discard,:) = [];
        allTestImgIds(discard) = [];
        allInstancetNames(discard) = [];
        allViewNames(discard) = [];

        % create output folder
        if ~exist(outputpath,'dir'), mkdir(outputpath); end
        delete([outputpath '/*.jpg']);
        delete([outputpath '/viewer.html']);
        
        % pick top detections only
        fprintf('picking top detections...\n');
        nExamples = min(nExamples,size(allBb2dDraw,1));
        topbb3d = allbb3d(1:nExamples,:);
        topbb3dtight = allbb3dtight(1:nExamples,:);
        topbb3d_fea = allbb3d_fea(1:nExamples,:);
        topBb2dDraw = allBb2dDraw(1:nExamples,:);
        topTestImgIds = allTestImgIds(1:nExamples);
        topInstancetNames = allInstancetNames(1:nExamples);
        topViewNames = allViewNames(1:nExamples);
       %saveToresult
       % bbStructs = formatbb3d(allbb3d,categName,allTestImgIds);
       save(['/n/fs/modelnet/Output/DetectionMat/' categName '.mat'],'allbb3d','allbb3dtight','allTestImgIds');
       % save masked images
       fprintf('generating output images...\n');
       maskedImageNames = cell(nExamples,1);
       for i = 1:nExamples,
            maskedImageNames{i} = sprintf('rank%03d-image%05d.jpg',i,topTestImgIds(i));   
            bbColor = [0 1 0];
            try
                tmp = load([pointFolder '/feature_' num2str(topTestImgIds(i)) '.mat'],'XYZworldframeTest','SpaceTest','Rtilt','Rorg','imgDepth');
                rgbImge =getImagesc(tmp.imgDepth);
                tmp.rgbTest = reshape(rgbImge,[],3);
                Rtilt  = tmp.Rtilt;
                %Rtilt = decomposeR (tmp.Rorg([1,3,2],:))*[1,0,0;0,0,1;0,1,0]*[1,0,0;0,-1,0;0,0,1]*tmp.Rorg*[1,0,0;0,-1,0;0,0,1]*[1,0,0;0,0,1;0,1,0];
                modeltmp = load([modelImageFolder '/' categName '/' topInstancetNames{i} '/' topViewNames{i} '.mat']);            
                [maskedImg,h,w] = applyBbMask(tmp.XYZworldframeTest,tmp.rgbTest,topbb3d(i,:),Rtilt,[0 1 0],modeltmp);
                %maskedImg = drawBbOnImage(maskedImg,topBb2dDraw(i,:),bbColor);         
                imwrite(maskedImg,sprintf('%s/%s',outputpath,maskedImageNames{i}));
            catch
                imageName = otherNegfiles{topTestImgIds(i)-500};
                imageName =imageName(find(imageName=='/',1,'last')+1:end-4);
                for fid =1:length(pointFolder), 
                    try room = load([pointFolder{fid} '/' imageName '.mat']);end
                end
                roomView =room.roomView;
                rgbImge =getImagesc(room.roomView.depth);
                modeltmp = load([modelImageFolder '/' categName '/' topInstancetNames{i} '/' topViewNames{i} '.mat']); 
                [rgbTest,XYZworldframeTest] = read_3d_pts_general(roomView.depth,roomView.K,size(roomView.depth),rgbImge,[1,1]);
                Rtilt = getRotationMatrix('tilt',roomView.Tilt);
                XYZworldframeTest = (Rtilt*XYZworldframeTest')';
                [maskedImg,h,w] = applyBbMask(XYZworldframeTest,rgbTest,topbb3d(i,:),Rtilt,[0 1 0],modeltmp);                
                maskedImg = drawBbOnImage(maskedImg,topBb2dDraw(i,:),bbColor);         
                imwrite(maskedImg,sprintf('%s/%s',outputpath,maskedImageNames{i}));
            end
        end
       
        %% create html viewer
        fprintf('generating html file...\n');
        htmlFullName = [outputpath '/amazing_viewer.html'];
        htmlFile = fopen(htmlFullName,'w');
        if htmlFile < 0, error('cannot create file %s.html',htmlFullName); end
        fprintf(htmlFile,'<html><head><title>3D Detection Viewer</title></head><body>\n');
        % script to hide model images
        fprintf(htmlFile,'<script language="JavaScript" type="text/JavaScript">\n');
        fprintf(htmlFile,'<!--\n');
        fprintf(htmlFile,'function toggleDiv(targetid){\n');
        fprintf(htmlFile,'var target=document.getElementById(targetid);\n');
        fprintf(htmlFile,'if (target.style.display !== ''none''){ target.style.display = ''none''; }\n');
        fprintf(htmlFile,'else { target.style.display = ''block''; }\n');
        fprintf(htmlFile,'}\n');
        fprintf(htmlFile,'-->\n');
        fprintf(htmlFile,'</script>\n');
        fprintf(htmlFile,'<style type="text/css">\n');
        fprintf(htmlFile,'<!--\n');
        fprintf(htmlFile,'#showModelImages { cursor: hand; cursor:pointer;}\n');
        fprintf(htmlFile,'.none { display: none; }\n');
        fprintf(htmlFile,'-->\n');
        fprintf(htmlFile,'</style>\n');
        
        % display header
        fprintf(htmlFile,'<center><h2>3D Detection Viewer<br>\n');
        fprintf(htmlFile,'%s\n',instanceName);
        if ~isempty(rString), fprintf(htmlFile,'_%s\n',rString); end
        if ~isempty(sString), fprintf(htmlFile,'_%s\n',sString); end
        if ~isempty(dString), fprintf(htmlFile,'_%s\n',dString); end
        if ~isempty(tString), fprintf(htmlFile,'_%s\n',tString); end
        fprintf(htmlFile,'</h2><center>\n');
        
        % show model images
        nImagesPerRow = 1;
        fprintf(htmlFile,'<div id="showModelImages" onClick="toggleDiv(''modelImagesDivId'')"><h3>1. Detection Models (click to show/hide)</h3></div>\n');
        fprintf(htmlFile,'<div id="modelImagesDivId"  class = "none">\n');
        fprintf(htmlFile,'<table style="overflow:hidden;" cellpadding="10">\n');
        fprintf(htmlFile,'<tr></tr>\n');
        nModels = length(allModelNames);
        relativeModelImageFolder = relativepath(modelImageFolder,outputpath);
        allModelViewNames = allViewNames;
        
        if showModel
            for entryIdx = 1:nModels,     
                try
                imgNameCurr = [relativeModelImageFolder '/' categName '/' allInstancetNames{i} '/' allModelViewNames{i} '.jpg'];
                img = imread([modelImageFolder '/' categName '/' allInstancetNames{i} '/' allModelViewNames{i} '.jpg']);
                width = size(img,2)/2;
                height = size(img,1)/2;
                fprintf(htmlFile,'\t<td><div style="position:relative;"><img src="%s" width="%d" height="%d" />\n',imgNameCurr,width,height);         
                fprintf(htmlFile,'\t<img src="%s" height="%d" />\n',[allModelViewNames{entryIdx} '_tpscores.jpg'],height);
                fprintf(htmlFile,'\t<img src="%s" height="%d" /></div></td>\n',[allModelViewNames{entryIdx} '_PR.jpg'],height);
                fprintf(htmlFile,'\t<img src="%s" height="%d" /></div></td>\n',[allModelViewNames{entryIdx} '_ifeature.jpg'],height);
                fprintf(htmlFile,'\t<img src="%s" height="%d" />\n',[allModelViewNames{entryIdx} '_mapping.jpg'],height);
                fprintf(htmlFile,'</tr>\n');
                end
                % show name
                 fprintf(htmlFile,'<tr>\n');
                 fprintf(htmlFile,'\t<td align="center"><a href="%s.ply">%s</a>\n<br>',allModelViewNames{entryIdx},allModelViewNames{entryIdx});
                 iterinfortmp = load([modelFolder '/' categName '/' topInstancetNames{entryIdx} '/' allModelViewNames{entryIdx} '.mat'],'Model');
                 iterinfor =iterinfortmp.Model.iter; 
                 fprintf(htmlFile,'<br>');
                 for iterid =1: size(iterinfor,1),
                    fprintf(htmlFile,'iter: %d. pSV: %d,nSV %d thr %.2f, +: %d, -: %d ,numNeg %d, numIm: %d<br>',iterinfor(iterid,1),iterinfor(iterid,2),iterinfor(iterid,3),iterinfor(iterid,4),iterinfor(iterid,5),iterinfor(iterid,6),iterinfor(iterid,7),iterinfor(iterid,8));
                 end  

                 try

                    [calibratefun,tpscores,fscores] = calibrationBaseOnPR(iterinfortmp.Model.oscfall,iterinfortmp.Model.socreall,1000);
                    f = figure('units','normalized','outerposition',[0 0 1 1],'Visible','Off');
                    plot(calibratefun(1,:), calibratefun(2,:), '.-'); hold on;
                    grid on; xlabel 'conf'; ylabel 'precision'; axis tight;
                    saveas(f,sprintf('%s/%s_mapping.jpg',outputpath,allModelViewNames{entryIdx}));
                    close(f);

                    f = figure('units','normalized','outerposition',[0 0 1 1],'Visible','Off');
                    plot(calibratefun(3,:), calibratefun(4,:), '.-'); 
                    hold on;grid on; xlabel 'recall'; ylabel 'precision'; axis equal; axis([0,1,0,1]);
                    saveas(f,sprintf('%s/%s_PR.jpg',outputpath,allModelViewNames{entryIdx}));
                    close(f);

                    f = figure('units','normalized','outerposition',[0 0 1 1],'Visible','Off');
                    plot(tpscores(:), '.-');
                    hold on;grid on; ylabel 'tpscores'; axis tight; 
                    thres =[0.1:0.1:0.6];
                    thrscore  = tpscores(ceil(thres*length(tpscores)));
                    plot([1:size(tpscores(:),1)],thrscore*ones(1,size(tpscores(:),1)));
                    sample =round(size(fscores(:),1)/length(tpscores));
                    plot(fscores(1:sample:size(fscores(:),1)),'.-r');
                    legend('pos','neg','0.1','0.2','0.3','0.5','0.6');
                    saveas(f,sprintf('%s/%s_tpscores.jpg',outputpath,allModelViewNames{entryIdx}));
                    close(f);

                end
                fprintf(htmlFile,'</td>');
                fprintf(htmlFile,'</tr>\n');
                 % insert an empty row for spacing
                fprintf(htmlFile,'<tr><td colspan="%d">&nbsp;<td></tr>\n',nImagesPerRow);
                
            end
        end
        fprintf(htmlFile,'</table>\n');
        fprintf(htmlFile,'</div>\n');
        % show detection results
        fprintf(htmlFile,'<h3>3. Detection results</h3>\n');
        fprintf(htmlFile,'<table style="overflow:hidden;" cellpadding="10">\n');
        fprintf(htmlFile,'<tr></tr>\n');
        img = imread([outputpath '/' maskedImageNames{(1)}]);
        width = round(viewScale * size(img,2));
        height = round(viewScale * size(img,1));
        for entryIdx =1:nExamples
            fprintf(htmlFile,'<tr>\n'); 
            fprintf(htmlFile,'\t<td><div style="position:relative;"><img src="%s" width="%d" height="%d" /></div></td>\n',maskedImageNames{entryIdx},width,height);
            fprintf(htmlFile,'</tr>\n');
       
            % show names
            fprintf(htmlFile,'<tr>\n');
            fprintf(htmlFile,'\t<td align="center"><a href="%s.ply">Rank%d (%.2f): Test_%05d-%s</a></td>\n',topViewNames{entryIdx},entryIdx,topbb3d(entryIdx,7),topTestImgIds(entryIdx),topViewNames{entryIdx});
            fprintf(htmlFile,'</tr>\n');
        end
            fprintf(htmlFile,'</table>\n');
    
        fprintf(htmlFile,'</body></html>\n');
        fclose(htmlFile);