function  [maxOverlaps,score,precision,recall,apScore]=getSingleModelAP(dataFolder,testIdRange,modelName,groundTruthBbs,gtLabels,goodGT,loadPrevMat)
             %% analyze model name
            [rIdx,rEIdx] = regexp([modelName '_'],'_r-?[.\d]+_');
            [sIdx,sEIdx] = regexp([modelName '_'],'_s-?[.\d]+_');
            [dIdx,dEIdx] = regexp([modelName '_'],'_d-?[.\d]+_');
            [tIdx,tEIdx] = regexp([modelName '_'],'_t-?[.\d]+_');
            [xIdx,xEIdx] = regexp([modelName '_'],'_x-?[.\d]+_');
            if length(rIdx) > 1 || length(sIdx) > 1 || length(dIdx) > 1 || length(tIdx) > 1 || length(xIdx) > 1, error('invalid model name'); end
            instanceNameEndIdx = length(modelName);
            if ~isempty(rIdx), instanceNameEndIdx = min(instanceNameEndIdx,rIdx-1); end
            if ~isempty(sIdx), instanceNameEndIdx = min(instanceNameEndIdx,sIdx-1); end
            if ~isempty(dIdx), instanceNameEndIdx = min(instanceNameEndIdx,dIdx-1); end
            if ~isempty(tIdx), instanceNameEndIdx = min(instanceNameEndIdx,tIdx-1); end
            if ~isempty(xIdx), instanceNameEndIdx = min(instanceNameEndIdx,xIdx-1); end
            rString = modelName(rIdx+1:rEIdx-1);
            sString = modelName(sIdx+1:sEIdx-1);
            dString = modelName(dIdx+1:dEIdx-1);
            tString = modelName(tIdx+1:tEIdx-1);
            xString = modelName(xIdx+1:xEIdx-1);
            instanceName = modelName(1:instanceNameEndIdx);
            idx = regexp(instanceName,'_(\d)*');
            
            if isempty(idx)
                categName = 'chair';
                instanceString = [];
            else
                categName = instanceName(1:idx-1);
                instanceString = instanceName(idx+1:end);
            end
      

            % find instances that matches model name
            instanceFolders = dir([dataFolder '/' categName '/']);
            instanceFolders = instanceFolders([instanceFolders.isdir]);
            keep = false(length(instanceFolders),1);
            for i = 1:length(instanceFolders),
                keep(i) = ~isempty(regexp(instanceFolders(i).name, instanceName, 'once'));
            end
            instanceFolders = instanceFolders(keep);
            if isempty(instanceFolders), error('no matched instance found'); end

            % find views that matches model name, and load mat files
            allModelInstanceNames = cell(0,0);
            allModelViewNames = cell(0,0);
            allBb3d = [];
            allBb2dDraw = [];
            allTestImgIds = [];
            allInstancetNameIds = [];
            allViewNameIds = [];
            allBb3dtight =[];
            for i = 1:length(instanceFolders),
                fprintf('In Single AP length(instanceFolders) %d:\n',length(instanceFolders) );
                viewFoldersCurr = dir([dataFolder '/' categName '/' instanceFolders(i).name '/']);
                viewFoldersCurr = viewFoldersCurr([viewFoldersCurr.isdir]);

                % pick view folders that matches input rotation and scale
                keep = false(length(viewFoldersCurr),1);
                parfor j = 1:length(viewFoldersCurr),
                    nameToMatch = [viewFoldersCurr(j).name '_'];
                    keep(j) = ~isempty(regexp(nameToMatch, instanceName, 'once'));
                    keep(j) = keep(j) && (isempty(rString) || ~isempty(regexp(nameToMatch, ['_' rString '_'], 'once')));
                    keep(j) = keep(j) && (isempty(sString) || ~isempty(regexp(nameToMatch, ['_' sString '_'], 'once')));
                    keep(j) = keep(j) && (isempty(dString) || ~isempty(regexp(nameToMatch, ['_' dString '_'], 'once')));
                    keep(j) = keep(j) && (isempty(tString) || ~isempty(regexp(nameToMatch, ['_' tString '_'], 'once')));
                    keep(j) = keep(j) && (isempty(xString) || ~isempty(regexp(nameToMatch, ['_' xString '_'], 'once')));
                end
                viewFoldersCurr = viewFoldersCurr(keep);

                % load mat files
                allModelInstanceNamesIdOff = length(allModelInstanceNames);
                allModelViewNamesIdOff = length(allModelViewNames);
                for j = 1:length(viewFoldersCurr),
                    allModelInstanceNames = [allModelInstanceNames; {instanceFolders(i).name}];
                    allModelViewNames = [allModelViewNames; {viewFoldersCurr(j).name}];                  
                    fprintf('In compute Single AP : loading results of %s...\n',viewFoldersCurr(j).name);
                    prevViewMat = [dataFolder '/' categName '/' instanceFolders(i).name '/' viewFoldersCurr(j).name '/viewTest.mat'];

                    if exist(prevViewMat,'file') && loadPrevMat,
                        fprintf('importing results of %s...\n',viewFoldersCurr(j).name);
                        prevResults = load(prevViewMat);
                        idx = ismember([prevResults.allTestImgIdsview],testIdRange);                
                        allBb3dview = prevResults.allBb3dview(idx,:);
                        allBb2dDrawview = prevResults.allBb2dDrawview(idx,:);
                        allBb3dtightview =prevResults.allBb3dtightview(idx,:);
                        allTestImgIdsview = prevResults.allTestImgIdsview(idx,:);
                        allInstancetNameIdsview =prevResults.allInstancetNameIdsview(idx,:);
                        allViewNameIdsview = prevResults.allViewNameIdsview(idx,:);
                                  
                    else
                        matFiles = dir([dataFolder '/' categName '/' instanceFolders(i).name '/' viewFoldersCurr(j).name '/*.mat']);                      
                        for k = 1:length(matFiles),
                            % parse mat file name
                            tmp = regexp(matFiles(k).name,'-','split');
                            testImgIdCurr = str2double(tmp{1}(5:end));
                            if isempty(find(testIdRange == testImgIdCurr,1)),continue;end

                            % load mat file
                            tmp = load([dataFolder '/' categName '/' instanceFolders(i).name '/' viewFoldersCurr(j).name '/' matFiles(k).name]);
                            bb3dCurr =tmp.result.bb3d;
                            bb2dDrawCurr = tmp.result.bb2dDraw;
                            allBb3dtightview = tmp.result.bbtight3d;
                            if size(bb3dCurr,1) ~= size(bb2dDrawCurr,1),
                                fprintf('\tInconsistent 2d and 3d bb size in %s! something is wrong. skipping this file...\n',matFiles(k).name);
                                continue;
                            end
                            allBb3d =[allBb3d; bb3dCurr];
                            allBb2dDraw = [allBb2dDraw; bb2dDrawCurr];
                            allBb3dtight =[allBb3dtight;allBb3dtightview];
                            allTestImgIds = [allTestImgIds; repmat(testImgIdCurr,[size(bb3dCurr,1) 1])];
                            allInstancetNameIds = [allInstancetNameIds; repmat(allModelInstanceNamesIdOff+j,[size(bb3dCurr,1) 1])];
                            allViewNameIds = [allViewNameIds; repmat(allModelViewNamesIdOff+j,[size(bb3dCurr,1) 1])];
                        end
                    end
                    
                    if exist(prevViewMat,'file') && loadPrevMat,
                        allBb3d =[allBb3d; allBb3dview];
                        allBb2dDraw = [allBb2dDraw; allBb2dDrawview];
                        allBb3dtight =[allBb3dtight;allBb3dtightview];
                        allTestImgIds = [allTestImgIds; allTestImgIdsview];
                        allInstancetNameIds = [allInstancetNameIds; allInstancetNameIdsview];
                        allViewNameIds = [allViewNameIds; allViewNameIdsview];
                    end
                end
            if isempty(allBb3d), error('no matched mat files found'); end

            % sort detections according to confidence
            fprintf('sorting detections...\n');
            [~,idx] = sort(allBb2dDraw(:,17),'descend');
            allBb3d = allBb3d(idx,:);
            allBb2dDraw = allBb2dDraw(idx,:);
            allBb3dtight = allBb3dtight(idx,:);
            allTestImgIds = allTestImgIds(idx);
            allInstancetNameIds = allInstancetNameIds(idx);
            allViewNameIds = allViewNameIds(idx);
            % non-max suppression
            uniqueTestImgIds = unique(allTestImgIds);
            discard = false(size(allBb3d,1),1);
            parfor i = 1:length(uniqueTestImgIds),
                idx = find(allTestImgIds == uniqueTestImgIds(i));
                pick = nmsMe_3d([allBb3d(idx,1:3),allBb3d(idx,1:3)+allBb3d(idx,4:6),allBb3d(idx,7)],0.3);
                discardCurr = false(size(allBb3d,1),1);
                discardCurr(idx(setdiff(1:length(idx),pick))) = true;
                discard = discard | discardCurr;
            end
            allBb3d(discard,:) = [];
            allBb2dDraw(discard,:) = [];
            allBb3dtight(discard,:) = [];
            allTestImgIds(discard) = [];
            allInstancetNameIds(discard) = [];
            allViewNameIds(discard) = [];

            % generate PR curve
            if ~isempty(groundTruthBbs),
                % pick ground truth of current class
                ConstsDetection;
                [~,classId] = ismember(categName,consts.classNames);
                isValidGt = ([groundTruthBbs.classId] == classId) & ismember([groundTruthBbs.imageNum],testIdRange);
                groundTruthBbs = groundTruthBbs(isValidGt);
                gtLabels = gtLabels(isValidGt);
                
                score =allBb2dDraw(:,17);
                if goodGT,
                    % AP for good ground truth
                    [apScore,precision,recall,isTp,isFp,isMissed,gtIdx,maxOverlaps,numOfgt,gtIdxall] = computePRCurveTightBB(categName,allBb3d,allBb3dtight,allTestImgIds,groundTruthBbs,gtLabels~=1);
                    %[apScore,precision,recall,isTp,isFp,isMissed,gtIdx,maxOverlaps,numOfgt] = computePRCurve(categName,allBb3d,allTestImgIds,groundTruthBbs,gtLabels~=1);
                else
                    [apScore,precision,recall,isTp,isFp,isMissed,gtIdx,maxOverlaps,numOfgt,gtIdxall] = computePRCurveTightBB(categName,allBb3d,allBb3dtight,allTestImgIds,groundTruthBbs,zeros(length(groundTruthBbs),1));
                end
            end
end