function [depth,K] = off2im_simple(offfile, ratio, xzRot, tilt, objz, objx,objy,modelsize)
% [depth,K] = off2im_simple('/Users/shurans/Documents/Modelnet/room_off/room_modified/bedroom000007.off', 1, 0, -0.25, 4, 0,-0.95,[3,3,3])
if ~exist('xzRot', 'var')
    xzRot =0;
end
if ~exist('tilt', 'var')
    tilt = - 0.1*pi;
end
if ~exist('objz', 'var')
    objz = 2 + rand() * 5;   % 2 to 5
end
if ~exist('objx', 'var')
    objx = (rand()*0.5-0.25) .* objz;
end
if ~exist('ratio', 'var')
    ratio = 1;
end
if ~exist('objy', 'var')
    objy = -1.3;
end
if ~exist('modelsize', 'var')
    modelsize = [1;1;1]*rand()*1.5+0.5;
end

    

%% Camera Paramter
imw = 640 * ratio; 
imh = 480 * ratio;
fx_rgb = 5.19e+02 * ratio;
fy_rgb = 5.19e+02 * ratio;
cx_rgb = imw/2;
cy_rgb = imh/2;
K=[fx_rgb 0 cx_rgb; 0 fy_rgb cy_rgb; 0 0 1];

C = [0;0;0];   

z_near = 0.3;
z_far_ratio = 1.2;
Ryzswi = [1, 0, 0; 0, 0, 1; 0, 1, 0];

%%
offobj = offLoader(offfile);
offobj.vmat = Ryzswi * offobj.vmat;
Robj = genRotMat(xzRot);
Rcam = genTiltMat(tilt);
P = K * Rcam * [eye(3), -C];

vmat = scalePoints(Robj*offobj.vmat, [objx;objy;objz], modelsize);

result = RenderMex(P, imw, imh, vmat, uint32(offobj.fmat))';
depth = z_near./(1-double(result)/2^32);
maxDepth = max(depth(abs(depth) < 100));
cropmask = (depth < z_near) | (depth > z_far_ratio * maxDepth);
depth(cropmask) =NaN;

end

function crop = findCropRegion(mask)

[xlist, ylist] = ind2sub(size(mask), find(mask));
xmin = min(xlist); xmax = max(xlist);
ymin = min(ylist); ymax = max(ylist);
crop = [xmin, ymin, xmax - xmin, ymax - ymin];

end

function R = genRotMat(theta)

R = [cos(theta), 0, -sin(theta)
    0, 1, 0
    sin(theta), 0, cos(theta)];

end

function R = genTiltMat(theta)

R = [1, 0, 0
    0, cos(theta), -sin(theta)
    0,  sin(theta), cos(theta)];

end

function coornew = scalePoints(coor, center, size)

% function coornew = scalePoints(coor, box)
% 
% parameters:
%   coor: 3*n coordinates of n points
%   center: 3*1 the center of new point cloud
%   size: 3*1 the size of new point cloud

minv = min(coor, [], 2);
maxv = max(coor, [], 2);
oldCenter = (minv+maxv)/2;
oldSize = maxv - minv;
%size defined by height
if isempty(size)
    scale =1;
else
    scale =size(2)./ oldSize(2);
end
%scale = min(size./ oldSize);
coornew = bsxfun(@plus, scale * coor, center-scale*oldCenter);
end
